%{

#include <iostream>
#include <string>
#include <parser/Parser.h>
#include <parser/Part.h>
#include <common/SyntaxTree.h>


extern parser::ParserContext* context;

#pragma warning(disable:4996)

#define LEX_RETURN(res)	{yylval.text = yytext; yylval.lineNumber = context->lineNumber; return res;}



#define YYSTYPE parser::Part

#include <parser/Parser_tab.h>

%}
ID [_a-zA-Z][_a-zA-Z0-9]*
DEC_NUMBER -?0|-?[1-9][0-9]*
HEX_NUMBER 0x[a-fA-F0-9]+ 
OCT_NUMBER 0[0-9]+
FLOAT_NUMBER -?[0-9]+.[0-9]+f?
%%
if																		LEX_RETURN(IFF)
else																	LEX_RETURN(ELSE)
while 																	LEX_RETURN(WHILE)
int 																	    LEX_RETURN(INT)
char																	   LEX_RETURN(CHAR)
long 																		LEX_RETURN(LONG)
short 																			LEX_RETURN(SHORT)
float 																		LEX_RETURN(FLOAT)
double 																		LEX_RETURN(DOUBLE)
void									LEX_RETURN(VOID)
bool									LEX_RETURN(BOOL)
true									LEX_RETURN(BOOLconstant)
false									LEX_RETURN(BOOLconstant)
return											LEX_RETURN(RETURN)		
\<\< 																LEX_RETURN(LS)
\>\> 																LEX_RETURN(RS)
\<= 				 LEX_RETURN(LE)
\>= 			LEX_RETURN(GE)
== 			LEX_RETURN(EQ)
!= 				LEX_RETURN(NE)
&& 			LEX_RETURN(ANDAND)
\|\| 			LEX_RETURN(OROR)
\*= 			LEX_RETURN(MULTassign)
\/= 			LEX_RETURN(DIVassign)
%= 			LEX_RETURN(MODassign)
\+= 				LEX_RETURN(PLUSassign)
\-= 				LEX_RETURN(MINUSassign)
\>\>= 				LEX_RETURN(RSassign)
\<\<= 				LEX_RETURN(LSassign)
\&= 				LEX_RETURN(ANDassign)
\|= 				LEX_RETURN(ORassign)
\^= 				LEX_RETURN(ERassign)
\+\+					LEX_RETURN(ICR)
\-\-				LEX_RETURN(DECR)
{ID}																	LEX_RETURN(IDENTIFIER)
{DEC_NUMBER}																LEX_RETURN(INTEGERconstant)
{HEX_NUMBER}																LEX_RETURN(HEXconstant)
{OCT_NUMBER}																LEX_RETURN(OCTALconstant)
{FLOAT_NUMBER}																LEX_RETURN(FLOATINGconstant)
['][^'][']																	LEX_RETURN(CHARACTERconstant)
\n																	{context->lineNumber++;}/*ignore eol*/;
[ \t]+																/*ignore whitespaces*/;
\/\/[^\n]*\n															{context->lineNumber++;}/*ignore comments*/
\/\*([^\*]|[\r\n]|(\*+([^\*\/]|[\r\n])))*\*+\/                         {
                                                                        for (int i = 0; i < strlen(yytext); i++) {
                                                                            if (yytext[i] == '\n') {
                                                                                context->lineNumber++;
                                                                            }
                                                                        }                                                                               
                                                                    }
.																	LEX_RETURN(yytext[0])
%%
