#ifndef __SYNTAX_2_AUTOMATA_H__
#define __SYNTAX_2_AUTOMATA_H__

#include <vector>

#include <common/AutomataSystem.h>
#include <common/SyntaxTree.h>
#include <common/DataModel.h>

namespace common {


struct AutomatonState;

}

namespace parser {
	struct ParserContext;
}


namespace s2a {

class AutomatonConverter {
private:
	enum TraverseType {
		TRAVERSE_LEFT_CHILD = 1,
		TRAVERSE_RIGHT_CHILD,
		TRAVERSE_BOTH_CHILDREN	
	};
	
	static const int IF_STATES = 2;
	static const int IF_WITH_ELSE_STATES = 4;
	static const int WHILE_STATES = 5;

	common::SyntaxTreeNode::NodeType lastNodeType;

	
	int stateNumber;	
	std::vector<int> _indexesOfReturnStates;
	std::string automatonName;

	common::DataModel dataModel;	
	std::vector<common::AutomatonState*> _directStates;
	std::vector<common::AutomatonState*> _reverseStates; 
	std::vector<common::Variable> _parameters;

	int countChildrenImpl(const common::SyntaxTreeNode* node, common::SyntaxTreeNode::NodeType& previousNodeType);
	int countChildren(const common::SyntaxTreeNode* node, TraverseType mode = TRAVERSE_BOTH_CHILDREN);
	bool hasGlobalDataModel;
	
	//////////////////////////////////////////
	//	some helper functions
	//////////////////////////////////////////////////////////////////////////
	void addStates();
	void addDefaultTransitions();
	void assignLineNumbers(const common::SyntaxTreeNode* node);
	void generateReturnStates();
public:
	
	AutomatonConverter(): stateNumber(0) {}

	void process(common::SyntaxTreeNode* root, bool hasGlobalData);
	common::Automaton* getAutomaton(const common::SyntaxTreeNode* root);

};


common::AutomataSystem* generateAutomataSystem(common::SyntaxTreeNode* node, parser::ParserContext* context);


} //namespace s2a

#endif //__SYNTAX_2_AUTOMATA_H__