#pragma once

#define DEBUG_SSTACK

#pragma warning (disable: 4996)

#include <stdio.h>
#include <windows.h>
#include <string>
#include <set>
#include <vector>
#include <map>

#include "..\..\interaction.h"

using std::string;

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

#pragma once

class CSStack;
class CSStackItem;

#include <stack>
#include "memory.h"
#include "stdlib.h"


using std::stack;


class CSStackItem
{
	friend class CSStack;

private:
	void	*data_;
	int		size_;	

public:
	explicit CSStackItem(void *data, int size)
	{
		size_ = size;
		data_ = data;
	}
};


class CSStack
{
private:
	stack<CSStackItem> stack_;
	int		pushesAssumed_;

public:
	/*CSStack();
	~CSStack();

	void	push(const void* data, int size);
	void	push(int i);
	void	push(bool b);

	void	pop(void *addr);
	bool	popBool();
	int		popInt();*/


	void	setPushesAssumed(int value)
	{
		if (value < 0)
		{
			char temp[64];
			sprintf(temp, "<Stack> Assumed pushes set to wrong value: %d", value);
			throw temp;
		}
		pushesAssumed_ = value;
	}



	CSStack()
	{
		pushesAssumed_ = 0;
	}

	~CSStack()
	{

	}


	void push(const void *data, int size)
	{
#ifdef DEBUG_SSTACK
		if (size == 4)
		{
			printf("Pushing item with size 4; value: %d\n", *((int *)data));
		}
		else
			printf("Pushing item with size %d\n", size);
#endif
		void *new_mem = malloc(size);
		if (new_mem == NULL)
		{
			throw "<Stack> Achtung! Memory failure! Please contact your PC manufacturer or distributor for support!";
		}
		memcpy(new_mem, data, size);
		CSStackItem cssi(new_mem, size);
		stack_.push(cssi);

		if (pushesAssumed_ > 0)
			pushesAssumed_--;
	}

	void push(int i)
	{
#ifdef DEBUG_SSTACK
		printf("Pushing int: %d    ", i);
#endif
		push(&i, sizeof(int));
	}

	void push(bool b)
	{
#ifdef DEBUG_SSTACK
		if (b) printf("Pushing bool: true    ");
		else printf("Pushing bool: false    ");
#endif
		push(&b, sizeof(bool));
	}

	void pop(void *addr)
	{
		if (stack_.empty())
		{
			throw "Achtung! Somehow you try to get the thing that doesn't exist! Please contact your doctor for support!";
		}
		CSStackItem cssi = stack_.top();
#ifdef DEBUG_SSTACK
		if (cssi.size_ == 4)
		{
			printf("Popping item with size 4; value: %d    ", *((int *)cssi.data_));
		}
		else
			printf("Popping item with size %d    ", cssi.size_);	
#endif
		stack_.pop();
		if (pushesAssumed_ >= 0)
		{
			if (pushesAssumed_ > 0)
			{
				printf("Won't pop this value due to assumed counter being %d", pushesAssumed_);
				pushesAssumed_--;
			}
			else
				memcpy(addr, cssi.data_, cssi.size_);
		}
		else 
			memcpy(addr, cssi.data_, cssi.size_);
		free(cssi.data_);
	}

	bool popBool()
	{
		bool result;
		CSStackItem cssi = stack_.top();
		if (cssi.size_ != sizeof(bool))
		{
			throw "Top item is not a boolean value which was assumed!";
		}
		pop(&result);
#ifdef DEBUG_SSTACK
		if (result) printf("Popping bool: true\n");
		else printf("Popping bool: false\n");
#endif
		return result;
	}

	int popInt()
	{
		int result;
		CSStackItem cssi = stack_.top();
		if (cssi.size_ != sizeof(int))
		{
			throw "Top item is not an integer value which was assumed!";
		}
		pop(&result);
#ifdef DEBUG_SSTACK
		printf("Popping int: %d\n", result);
#endif
		return result;
	}

};

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
/*         .  
*  ,   ,   .   
*    ,        
*
*/

#define PRIORITY_NORMAL	100
#define PRIORITY_NO_STOP 101

class AutomatonBase
{
private:
	
	std::set<int> _stateBreakpoints;

protected:
    
	std::map<int, std::vector<AutomatonBase*>> _childAutomata;
	int endState_;
	//  
	AutomatonBase *child_;
	
	int currentState_;
	int previousState_;
	
	CSStack *stack_;

	string name_;

	int* statePriorities_;


    bool mustBreakAtState(int stateNumber) { return _stateBreakpoints.find(stateNumber) != _stateBreakpoints.end(); }
    

public:
	explicit AutomatonBase(int endState, const string& name):endState_(endState),currentState_(0),name_(name)
	{
		statePriorities_ = new int[endState_+1];
		stack_ = new CSStack();
		child_ = NULL;
	}

	~AutomatonBase()
	{
		delete stack_;
		delete[] statePriorities_;
		for (std::map<int, std::vector<AutomatonBase*>>::iterator iter = _childAutomata.begin(); iter != _childAutomata.end(); ++iter) {
			for (std::vector<AutomatonBase*>::iterator iter2 = iter->second.begin(); iter2 != iter->second.end(); ++iter2) {
				delete *iter2;
			}
		}
		_childAutomata.clear();
	}

	virtual void stepForward(int stepNumber = 1) = 0;
	virtual string getVariable(const string& name) = 0;


	bool isAtStart() const		{return currentState_ == 0;}
	bool isAtEnd() const		{return currentState_ == endState_ - 1;}
	int getState() const			{return currentState_;}
	void setState(int newValue)	
	{
		previousState_ = currentState_;
		currentState_ = newValue;
	}
	int getMaxState() const		{return endState_;}
	string getName() const		{return name_;}
	AutomatonBase *getChild() const {return child_;}

	virtual void stepBackward(int stepNumber = 1)
	{
		//no code here. Not pure virtual so automata without
		//auto-reverse won't have to define this function
	}
	void toStart()
    {
        while (currentState_>0) {
            if (mustBreakAtState(currentState_)) {
                return;
            }
            stepBackward();
        }
    }

    void toEnd()
    {
        while (currentState_ < endState_ - 1) {
            if (mustBreakAtState(currentState_)) {
                return;
            }
            stepForward();
        }
    }

    void addStateBreakpoint(int stateNumber) { _stateBreakpoints.insert(stateNumber); }
    void removeStateBreakpoint(int stateNumber) { _stateBreakpoints.erase(stateNumber); }

};

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

string toString(bool b)
{
	if (b) return "true";
	else return "false";
}

string toString(char c)
{
	int i = c;
	char temp[32];
	sprintf(temp, "%d: '%c'", i, c);
	return temp;
}

string toString(unsigned char uc)
{
	int ui = uc;
	char temp[32];
	sprintf(temp, "%d: '%c'", ui, uc);
	return temp;
}

string toString(short uc)
{
	int i = uc;
	char temp[32];
	sprintf(temp, "%d", i);
	return temp;
}

string toString(unsigned short uc)
{
	unsigned int ui = uc;
	char temp[32];
	sprintf(temp, "%u", ui);
	return temp;
}

string toString(int i)
{
	char temp[32];
	sprintf(temp, "%d", i);
	return temp;
}

string toString(unsigned int ui)
{
	char temp[32];
	sprintf(temp, "%u", ui);
	return temp;
}

string toString(float f)
{
	char temp[32];
	sprintf(temp, "%f", f);
	return temp;
}

string toString(double d)
{
	char temp[32];
	sprintf(temp, "%lf", d);
	return temp;
}

string toString(void* p)
{
	char temp[32];
	sprintf(temp, "%08x", (unsigned int) p);
	return temp;
}