#include <stdio.h>

#include <codegen/ADBBuilder.h>
#include <codegen/PDGBuilder.h>

#include <common/AutomataSystem.h>
#include <common/Automata.h>

#include <autolog/autoLog.h>

const char resultFileName[] = "output\\Result.adb";

const int MAGIC_NUMBER = 0x0ADBF173;

//////////////////////////////////////////////////////////////////////////
//
// .adb is a binary file. Its structure is:
//
//  Header: 
//		magic number
//		path to source file
//		overall number of automata
//  For each automaton:
//		name
//		number of states
//		state number - first source line - last source line
//
//	if line number is < 0 than this state has priority PRIORITY_NO_STOP and
//  shall not be displayed
//
//////////////////////////////////////////////////////////////////////////
namespace {

void generateAutomatonInfo(FILE* outputFile, const common::Automaton* automaton)
{

	unsigned int strLength = automaton->name.length();
	fwrite(&strLength, sizeof(unsigned int), 1, outputFile);
	fwrite(automaton->name.c_str(), sizeof(char), automaton->name.length(), outputFile);
	fwrite(&automaton->endState, sizeof(int), 1, outputFile);

	for (int i = 0; i < automaton->endState; i++)	{
		fwrite(&automaton->states[i]->firstSourceString, sizeof(int), 1, outputFile);
		fwrite(&automaton->states[i]->lastSourceString, sizeof(int), 1, outputFile);
	}
}


} // anonymous namespace

void output::generateDebugInfoFile( const common::AutomataSystem* system , const char* sourceFileName)
{
	FILE* outputFile = fopen(resultFileName, "wb");
	if (!outputFile) {
		autoLog(Log::MSG_ERROR, "ADB Builder", "Failed to open file for writing");
		return;
	}
	fwrite(&MAGIC_NUMBER, sizeof(int), 1, outputFile);

	unsigned int strLength = strlen(sourceFileName);
	fwrite(&strLength, sizeof(unsigned int), 1, outputFile);
	fwrite(sourceFileName, sizeof(char), strlen(sourceFileName), outputFile);

	int size = system->automata.size();
	
	fwrite(&size, sizeof(size_t), 1, outputFile);
	for (unsigned int i = 0; i < system->automata.size(); i++) {
		::generateAutomatonInfo(outputFile, system->automata[i]);
        outputPDGToFile(outputFile, system->dependencyGraphs[i]);
	}
     
	fclose(outputFile);
}