#include <fstream>
#include <vector>

#include <codegen/AutomataOutput.h>
#include <common/AutomataSystem.h>
#include <common/Automata.h>
#include <common/DataModel.h>
#include <autolog/autoLog.h>

#include <codegen/SkeletonParser.h>

using namespace output;

#define TAB "\t"



common::Automaton* currentAutomaton;
const common::AutomataSystem* currentAutomatoaSystem;

void nameOutput(std::fstream& out)
{
	out<<currentAutomaton->name;
}

void constructorOutput(std::fstream& output)
{
		
    std::string parameterList;
    for (unsigned int i = 0; i < currentAutomaton->parameters.size(); i++) {
        if ( i > 0) {
            parameterList.append(", ");
        }
        parameterList.append(currentAutomaton->parameters[i].getDefinition());
        
    }

	output<<currentAutomaton->name<<"Automaton::"<<currentAutomaton->name<<"Automaton("<<parameterList<<") : AutomatonBase("<<currentAutomaton->endState<<",\""<<currentAutomaton->name<<"\")\n";
	output<<"{\n";



	output<<TAB<<"data = new "<<currentAutomaton->name<<"DataModel;\n";

    for (unsigned int i = 0; i < currentAutomaton->parameters.size(); i++) {
       output<<TAB<<"data->"<<currentAutomaton->parameters[i].getName()<<" = "<<currentAutomaton->parameters[i].getName()<<";\n";
    }

	for (int i = 0; i < currentAutomaton->endState; i++)
	{
		output<<TAB<<"statePriorities_["<<i<<"] = "<< currentAutomaton->states[i]->priority<<";\n";
	}


	output<<"}\n";
}

void parametersOutput(std::fstream& output)
{
    for (unsigned int i = 0; i < currentAutomaton->parameters.size(); i++) {
        if ( i > 0) {
            output<<", ";
        }
        output<<currentAutomaton->parameters[i].getDefinition();

    }
}

void directOutput(std::fstream& output)
{
	output<<"//First, follow transition from current to next state \n";
	output<<TAB<<"switch(currentState_){\n";
	for (int i = 0; i < currentAutomaton->endState;i++) {
		output<<TAB<<TAB<<"case "<<i<<": \n";

		for (unsigned int j = 0; j < currentAutomaton->states[i]->transitions.size(); j++) {

			if(currentAutomaton->states[i]->transitions[j]->isConditional) {
				output<<TAB<<TAB<<TAB<<"if("<<currentAutomaton->states[i]->transitions[j]->condition<<")\n";
				
				if (currentAutomaton->states[i]->transitions[j]->strTo.length() > 0) {
					output<<TAB<<TAB<<TAB<<"{\n"<<TAB<<TAB<<TAB<<TAB<<" setState("<<currentAutomaton->states[i]->transitions[j]->strTo<<");\n";
				} else {
					output<<TAB<<TAB<<TAB<<"{\n"<<TAB<<TAB<<TAB<<TAB<<" setState("<<currentAutomaton->states[i]->transitions[j]->to<<");\n";
				}
				
				for (unsigned int k = 0; k < currentAutomaton->states[i]->transitions[j]->_action.size(); k++) {
					output<<TAB<<TAB<<TAB<<TAB<<currentAutomaton->states[i]->transitions[j]->_action[k]<<";\n";
				}
				output<<TAB<<TAB<<TAB<<"} \n";
			}
			else {
				if (currentAutomaton->states[i]->transitions[j]->strTo.length() > 0) {
					output<<TAB<<TAB<<TAB<<"{\n"<<TAB<<TAB<<TAB<<TAB<<" setState("<<currentAutomaton->states[i]->transitions[j]->strTo<<");\n";
				} else {
					output<<TAB<<TAB<<TAB<<"{\n"<<TAB<<TAB<<TAB<<TAB<<" setState("<<currentAutomaton->states[i]->transitions[j]->to<<");\n";
				}
				output<<TAB<<TAB<<TAB<<"} \n";

				for (unsigned int k = 0; k < currentAutomaton->states[i]->transitions[j]->_action.size(); k++) {
					output<<TAB<<TAB<<TAB<<currentAutomaton->states[i]->transitions[j]->_action[k]<<";\n";
				}
			}
		}

		output<<TAB<<TAB<<"break; \n";
	}
	/*
	output<<TAB<<TAB<<"case "<<currentAutomaton->endState<<": \n";
		output<<TAB<<TAB<<TAB<<"//This is final state, no action here now \n";
		output<<TAB<<TAB<<"break; \n";*/
	
	output<<TAB<<TAB<<"default:\n";
	output<<TAB<<TAB<<TAB<<"//Enter some error notification code here \n";
	output<<TAB<<TAB<<"break; \n";
	output<<TAB<<"} \n";

	output<<"//Now do all necessary actions in the state. \n";
	output<<TAB<<"switch(currentState_){\n";

	for (int i = 0;i < currentAutomaton->endState; i++) {
		output<<TAB<<TAB<<"case "<<i<<": \n";
		if (currentAutomaton->states[i]->pushesAssumed > 0) {
			output<<TAB<<TAB<<"stack_->setPushesAssumed("<<currentAutomaton->states[i]->pushesAssumed<<");\n";
		}

		for (unsigned int j = 0; j < currentAutomaton->states[i]->actions.size(); j++) {
			output<<TAB<<TAB<<TAB<<currentAutomaton->states[i]->actions[j]<<"; \n";

		}
		output<<TAB<<TAB<<"break; \n";
	}
/*
	output<<TAB<<TAB<<"case "<<currentAutomaton->endState<<": \n";
	output<<TAB<<TAB<<TAB<<"//This is final state, no action here now \n";
	output<<TAB<<TAB<<"break; \n";*/

	output<<TAB<<TAB<<"default:\n";
	output<<TAB<<TAB<<TAB<<"//Enter some error notification code here \n";
	output<<TAB<<TAB<<"break; \n";
	output<<TAB<<"}\n";


}

void reverseOutput(std::fstream& output)
{
	output<<TAB<<"switch(currentState_){\n";
	for (int i = 0; i < currentAutomaton->endState; i++) {

		output<<TAB<<TAB<<"case "<<i<<": \n";

		for (int j=currentAutomaton->reverseStates[i]->actions.size() - 1; j >= 0; j--) {
			output<<TAB<<TAB<<TAB<<currentAutomaton->reverseStates[i]->actions[j]<<"; \n";
		}

		output<<TAB<<TAB<<"break; \n";
	}
	/*
	output<<TAB<<TAB<<"case "<<currentAutomaton->endState<<": \n";
		output<<TAB<<TAB<<TAB<<"//This is final state, no action here now \n";
		output<<TAB<<TAB<<"break; \n";*/
	
	output<<TAB<<TAB<<"default:\n";
	output<<TAB<<TAB<<TAB<<"//Enter some error notification code here \n";
	output<<TAB<<TAB<<"break; \n";
	output<<TAB<<"} \n";
	output<<"//Now step one state back \n";

	output<<TAB<<"switch(currentState_){\n";
	for (int i = 0; i < currentAutomaton->endState; i++) {
		output<<TAB<<TAB<<"case "<<i<<": \n";

		for (unsigned int j = 0; j < currentAutomaton->reverseStates[i]->transitions.size(); j++)	{
			if(currentAutomaton->reverseStates[i]->transitions[j]->isConditional) {
			
				output<<TAB<<TAB<<TAB<<"if("<<currentAutomaton->reverseStates[i]->transitions[j]->condition<<")\n";

				if (currentAutomaton->reverseStates[i]->transitions[j]->strTo.length() > 0) {
					output<<TAB<<TAB<<TAB<<"{\n"<<TAB<<TAB<<TAB<<TAB<<" setState("<<currentAutomaton->reverseStates[i]->transitions[j]->strTo<<");\n";
				} else {
					output<<TAB<<TAB<<TAB<<"{\n"<<TAB<<TAB<<TAB<<TAB<<" setState("<<currentAutomaton->reverseStates[i]->transitions[j]->to<<");\n";
				}

				for (unsigned int k = 0; k < currentAutomaton->reverseStates[i]->transitions[j]->_action.size(); k++) {
					output<<TAB<<TAB<<TAB<<TAB<<currentAutomaton->reverseStates[i]->transitions[j]->_action[k]<<";\n";
				}
				output<<TAB<<TAB<<TAB<<"} \n";
			}
			else {
				if (currentAutomaton->reverseStates[i]->transitions[j]->strTo.length() > 0) {
					output<<TAB<<TAB<<TAB<<"{\n"<<TAB<<TAB<<TAB<<TAB<<" setState("<<currentAutomaton->reverseStates[i]->transitions[j]->strTo<<");\n";
				} else {
					output<<TAB<<TAB<<TAB<<"{\n"<<TAB<<TAB<<TAB<<TAB<<" setState("<<currentAutomaton->reverseStates[i]->transitions[j]->to<<");\n";
				}
				output<<TAB<<TAB<<TAB<<"} \n";
			}
		}

		output<<TAB<<TAB<<"break; \n";
	}
/*
	output<<TAB<<TAB<<"case "<<currentAutomaton->endState<<": \n";
	output<<TAB<<TAB<<TAB<<"setState(stack_->popInt()); //This is final state,\n";
	output<<TAB<<TAB<<"break; \n";*/

	output<<TAB<<TAB<<"default:\n";
	output<<TAB<<TAB<<TAB<<"//Enter some error notification code here \n";
	output<<TAB<<TAB<<"break; \n";
	output<<TAB<<"} \n";


}

void outputMultipleAutomata(std::fstream& output)
{

	CSkeletonParser parser;
	parser.registerKeyword("NAME",&nameOutput);
	parser.registerKeyword("CONSTRUCTOR",&constructorOutput);
	parser.registerKeyword("FORWARD",&directOutput);
	parser.registerKeyword("BACKWARD",&reverseOutput);
    parser.registerKeyword("PARAMETERS", &parametersOutput);

	for (unsigned int i = 0; i < currentAutomatoaSystem->automata.size(); i++) {
		currentAutomaton = currentAutomatoaSystem->automata[i];
		parser.parse("Automaton_Skeleton.txt",output);

	}
}

void outputDataIncludes(std::fstream& output)
{
	if (currentAutomatoaSystem->globalDataModel) {
		output<<"#include \"GlobalDataModel.h\"\n";
	}
	for (unsigned int i = 0; i < currentAutomatoaSystem->automata.size(); i++) {
		output<<"#include \""<<currentAutomatoaSystem->automata[i]->data.name<<"DataModel.h\"\n";
	}


}

void globalDataDefine(std::fstream& output)
{
	output<<"GlobalDataModel* globalData = new GlobalDataModel();\n";
}

void outputGlobalWatchReturn(std::fstream& output)
{
	output<<"\tif(result == \"Unrecognized variable name.\")return globalData->getVariable(name); \n";
}


int output::generateCode(const common::AutomataSystem* automataSystem )
{
	output::CSkeletonParser parser;
	currentAutomatoaSystem = automataSystem;
	parser.registerKeyword("AUTOMATA",&outputMultipleAutomata);
	parser.registerKeyword("DATA_INCLUDES",&outputDataIncludes);
	if (currentAutomatoaSystem->globalDataModel)
	{
		
		parser.registerKeyword("RETURN_FROM_GLOBAL", &outputGlobalWatchReturn);
		parser.registerKeyword("GLOBAL_DATA_MODEL_DEFINITION",&globalDataDefine);
	}
#ifdef TRACER_V2
    parser.parse("Skeleton_v2.txt","Output\\Result.cpp");
#else
	parser.parse("Skeleton.txt","Output\\Result.cpp");
#endif

	std::string dataModelFileName;
	for (unsigned int i = 0; i < currentAutomatoaSystem->automata.size(); i++) {
		dataModelFileName = "output\\";
		dataModelFileName.append(currentAutomatoaSystem->automata[i]->data.name).append("DataModel.h");
		currentAutomatoaSystem->automata[i]->data.writeToFile(dataModelFileName);
	}
    if (currentAutomatoaSystem->globalDataModel) {
        currentAutomatoaSystem->globalDataModel->writeToFile("output\\GlobalDataModel.h");
    }
	autoLog(Log::MSG_REPORT, "Code Generation", "Code generation completed succesfully");
	return 0;
}