#ifndef __AUTOMATA_H__
#define __AUTOMATA_H__

#include <string>
#include <vector>

#include <common/Variable.h>
#include <common/DataModel.h>
#include <common/AutomatonStateData.h>

namespace common {
/*,   */
	struct Transition
	{
		/*   */
		int to;
		std::string strTo;
		/*  */
		std::string condition;
		/*    */
		bool isConditional;
		//     
		std::vector<std::string> _action;

		Transition():to(-1),condition(""),isConditional(false){}

		Transition(int a, const std::string& b, bool c, const std::string& d = ""):to(a),condition(b),isConditional(c)
		{
			if (d.length() > 0) {
				_action.push_back(d);
			}
		}
		Transition(const std::string& a, const std::string& b, bool c, const std::string& d = ""):strTo(a),condition(b),isConditional(c)
		{
			if (d.length() > 0) {
				_action.push_back(d);
			}
		}

		void setAllValues(int a, const std::string& b, bool c, const std::string& d = ""){
			to = a;
			condition = b;
			isConditional = c;
			if (d.length() > 0) {
				_action.push_back(d);
			}
		}

	};
// ,     
	struct AutomatonState
	{
		int stateNumber;

		/*   .
			    PRIORITY_NO_STOP
			        
			   (      )
		*/
		enum Priority {
			PRIORITY_NORMAL = 100,
			PRIORITY_NO_STOP
		} priority;
		common::AutomatonStateData* stateData;
		std::vector<std::string> actions; //  ,     
		std::vector<Transition*> transitions; //   
		int pushesAssumed; //        

		int firstSourceString;
		int lastSourceString;

		AutomatonState():stateNumber(0),pushesAssumed(0), stateData(NULL)
		{
		}

		AutomatonState(int number,Priority priority = PRIORITY_NORMAL):stateNumber(number),pushesAssumed(0),priority(priority), stateData(NULL)
		{
		}

	};


/*         */
	struct Automaton
	{
		std::string name;
		int endState;
		AutomatonState** states;
		AutomatonState** reverseStates;
		std::vector<common::Variable> parameters; //  ,    
		common::DataModel data; //      
        
        std::string returnValue;

		Automaton():states(NULL),reverseStates(NULL),name("Result"){}
		Automaton(const std::string &name):states(NULL),reverseStates(NULL),name(name){}

		~Automaton()
		{
						
				delete[] states;
			if (reverseStates!=NULL)
			{
				delete[] reverseStates;
			}
		}

	};


} //namespace common

#endif //__AUTOMATA_H__