#include <fstream>

#include <common/DataModel.h>
#include <common/Variable.h>
#include <CodeGen/SkeletonParser.h>
#include <autolog/autoLog.h>
#include <Syntax2Automata/SyntaxHelpers.h>


using namespace common;
using namespace output;

#define TAB "\t"

DataModel* currentDataModel;

namespace {

void GenerateWatchFunction(std::fstream& output)
{
	for (unsigned int i = 0; i < currentDataModel->variables.size(); i++) {	
		
		output<<TAB<<TAB<<"if (variable_name == \""<<currentDataModel->variables[i].getName()<<"\") {\n";

		if (currentDataModel->variables[i].getTypeId().getDimensions() > 0) {
			output<<TAB<<TAB<<TAB<<"return \"Type: \" + string(\""<<currentDataModel->variables[i].getType()<<"\").append(\"; Value: \")";
			for (unsigned int j = 0; j < currentDataModel->variables[i].getTypeId().getCheToTam(0); j++) {
				output<<".append(toString("<<currentDataModel->variables[i].getName()<<"[" <<j<<"])).append(\" \")";
			}
			output<<";\n";
		} else {
#ifdef TRACER_V2
			output<<TAB<<TAB<<TAB<<"return VariableDesc("<<currentDataModel->variables[i].getType()<<", variable_name, toString("<<currentDataModel->variables[i].getName()<<")); \n";
#else
			output<<TAB<<TAB<<TAB<<"return \"Type: \" + string(\""<<currentDataModel->variables[i].getType()<<"\").append(\"; Value: \").append(toString("<<currentDataModel->variables[i].getName()<<")); \n";
#endif
		}
		output<<TAB<<TAB<<"}\n";
	
	}
}

void GenerateName(std::fstream& output)
{
	output<<currentDataModel->name;
}

void GeneratepVariablesDefinitions(std::fstream& output)
{
	for (unsigned int i = 0; i < currentDataModel->variables.size(); i++) {
		//output<<"\t"<<currentDataModel->variables[i].getType()<<" "<<currentDataModel->variables[i].getName()<<";\n";
		std::string s = currentDataModel->variables[i].getDefinition();
		int pos = s.find("=");
		if (pos !=std::string::npos) {
			s.erase(pos, s.length() - pos);
		}
		output<<"\t"<<s<<";\n";
	}
}

void GenerateDefaultCtor(std::fstream& output)
{
	for (unsigned int i = 0; i < currentDataModel->variables.size(); i++) {
        if (currentDataModel->variables[i].hasInitializer()) {
			std::string s = currentDataModel->variables[i].getInitializer();
			s2a::cleanString(s);
            output<<"\t"<<currentDataModel->variables[i].getName()<<" = "<<s<<";\n";
        }
    }

}

void GenerateCopyCtor(std::fstream& output)
{
	for (unsigned int i =0 ;i < currentDataModel->variables.size(); i++)	{
		if (currentDataModel->variables[i].getTypeId().getDimensions() == 0) {
			output<<TAB<<currentDataModel->variables[i].getName()<<" = "<<"dm."<<currentDataModel->variables[i].getName()<<";\n";
		}
	}
}


void GenerateSaveToStackFunction(std::fstream& output)
{
	for (unsigned int i = 0; i < currentDataModel->variables.size(); i++)	{

		output<<"\t\tstack->push(&"<<currentDataModel->variables[i].getName()<<",sizeof("<<currentDataModel->variables[i].getName()<<"));\n";

	}
}

void GenerateLoadFromStackFunction(std::fstream& output)
{
	for (int i = currentDataModel->variables.size()-1; i >= 0; i--) {
		output<<"\t\tstack->pop(&"<<currentDataModel->variables[i].getName()<<");\n";
	}
}



} //anonymous namespace


void DataModel::writeToFile(const std::string& fileName)
{
	CSkeletonParser parser;
	parser.registerKeyword("NAME",&::GenerateName);
	parser.registerKeyword("VARS",&::GeneratepVariablesDefinitions);
	parser.registerKeyword("VAR_OUTPUT",&::GenerateWatchFunction);
	parser.registerKeyword("DEFAULT_CTOR",&::GenerateDefaultCtor);
	parser.registerKeyword("COPY_CTOR",&::GenerateCopyCtor);
	parser.registerKeyword("STACK_SAVE",&::GenerateSaveToStackFunction);
	parser.registerKeyword("STACK_LOAD",&::GenerateLoadFromStackFunction);
	

	currentDataModel = this;
	parser.parse("DataModel_Skeleton.txt", fileName.c_str());
	autoLog(Log::MSG_REPORT, "Data Model Builder", "Data model " + name + " created successfully");

}

bool DataModel::contains(const std::string& varName)
{
    for (unsigned int i = 0; i < variables.size(); ++i) {
        if (variables[i].getName() == varName) {
            return true;
        }
    }

    return false;
}

void DataModel::cleanVariableNames()
{
	for (unsigned int i = 0; i < variables.size(); i++)	{
		variables[i].cleanName();
	}
}