#include <stdio.h>

#include <Parser/Parser.h>
#include <Syntax2Automata/Syntax2Automata.h>
#include <autolog/autoLog.h>
#include <common/AutomataSystem.h>
#include <codegen/AutomataOutput.h>
#include <codegen/ADBBuilder.h>
#include <slicing/ProgramDependencyGraph.h>


//Log class instance
Log log;

#define ABORT {\
                  criticalError();\
                  return -1;\
              }


#define FAIL_CHECK(cond) {\
                            if (!cond) {\
                                ABORT\
                            }\
                          }

void criticalError()
{
    autoLog(Log::MSG_FATAL, "Automata Konverter", "Critical error, build aborted.\n Press enter.");
    getc(stdin);
        
}

int main(int argc, char *argv[])
{
	parser::ParserContext context;
	common::AutomataSystem* system;
	
	Log::getInstance()->logMessage(Log::MSG_REPORT, "Automata Konverter", "Build started");

	common::SyntaxTreeNode* root = parser::generateSyntaxTree(argv[1], &context);
FAIL_CHECK(root);
	system = s2a::generateAutomataSystem(root, &context);
FAIL_CHECK(system);
	output::generateCode(system);
    output::generateAutomataSystemSchemes(system);

    system->buildPDG();

	output::generateDebugInfoFile(system, argv[1]);

	autoLog(Log::MSG_REPORT, "Automata Konverter", "Build Finished");

	return 0;
}