#include <common/comm.h>
#include <slicing/ProgramDependencyGraph.h>
#include <map>

struct controlFlowStackItem{
    int stateNumber;
    int validTo;
    controlFlowStackItem(int s, int v):stateNumber(s), validTo(v){}
};



void validateControlFlowStack(std::vector<controlFlowStackItem>& v, int i)
{
    bool flag = v.size() > 0;
    while (flag)
    {
        if (v[v.size() - 1].validTo <= i)
        {
            v.pop_back();
            flag = v.size() > 0;
        } else {
            flag = false;
        }
       
    }
}

bool slicing::ProgramDependencyGraph::create( common::Automaton* automaton )
{
    int currentIndex = 0;
    GraphNode* node;

  //  std::vector<int> controlFlowStack;


    for (int i = 0; i < automaton->endState; i++)
    {
        common::AutomatonState* state = automaton->states[i];
        
        if (!state->stateData || state->stateData->stateType == common::AutomatonStateData::STATE_TYPE_INTERNAL || state->stateData->stateType == common::AutomatonStateData::STATE_TYPE_START_END)
        {
            continue;
        }

        if(state->stateData->stateType == common::AutomatonStateData::STATE_TYPE_OPERATOR) {

            
           /* for (unsigned int j = 0; j < state->actions.size(); j++)
                {
                    if (automaton->states[i]->actions[j].find("stack_->") != std::string::npos)
                    {
                        continue;
                    }*/
                    node = new GraphNode;
                    node->nodeNumber = currentIndex++;
                    node->stateNumber = i;
                    node->lineStart = automaton->states[i]->firstSourceString;
                    node->lineEnd = automaton->states[i]->lastSourceString;
                   // node->statement = automaton->states[i]->actions[j];
                    node->isControlFlowStatement = false;

            

                _nodes.push_back(node);
                //}
        } else if (state->stateData->stateType = common::AutomatonStateData::STATE_TYPE_CONDITIONAL)
        {
            node = new GraphNode;
            node->nodeNumber = currentIndex++;
            node->stateNumber = i;
            node->lineStart = automaton->states[i]->firstSourceString;
            node->lineEnd = automaton->states[i]->lastSourceString;
            node->isControlFlowStatement = true;
            
            _nodes.push_back(node);
        }

        
    }
    buildControlFlowEdges(automaton);
    buildDataFlowEdges(automaton);
    return true;

}

void slicing::ProgramDependencyGraph::buildControlFlowEdges(common::Automaton* automaton)
{
    std::vector<controlFlowStackItem> controlFlowStack;

    for (unsigned int i = 0 ; i < _nodes.size(); i++) {

        validateControlFlowStack(controlFlowStack, i);

        if (controlFlowStack.size() > 0)
        {
            _nodes[i]->controlDependentEdges.push_back(controlFlowStack[controlFlowStack.size() - 1].stateNumber);
        }
        if (_nodes[i]->isControlFlowStatement)
        {
            for (int j = 0; j < automaton->states[_nodes[i]->stateNumber]->stateData->subordinateGroups; j++) {
                controlFlowStack.push_back(controlFlowStackItem(i, automaton->states[_nodes[i]->stateNumber]->stateData->subordinateStates[j]));
            }
        }
    }
}

void slicing::ProgramDependencyGraph::buildDataFlowEdges( common::Automaton* automaton )
{
    std::map<std::string, int> definitionMap;
     for (unsigned int i = 0 ; i < _nodes.size(); i++) {
        for (unsigned int j = 0; j < automaton->states[_nodes[i]->stateNumber]->stateData->dependsOn.size(); ++j) {
            if (definitionMap.find(automaton->states[_nodes[i]->stateNumber]->stateData->dependsOn[j])!= definitionMap.end())
            {
                _nodes[i]->dataDependentEdges.push_back(definitionMap[automaton->states[_nodes[i]->stateNumber]->stateData->dependsOn[j]]);
            }
        }
        for (unsigned int j = 0; j < automaton->states[_nodes[i]->stateNumber]->stateData->changeSet.size(); ++j) {
            definitionMap[automaton->states[_nodes[i]->stateNumber]->stateData->changeSet[j]] = i;
        }

     }
}