#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/TestFixture.h>

#include <common/Variable.h>

class VariableTest: public CPPUNIT_NS::TestFixture {

	CPPUNIT_TEST_SUITE( VariableTest );
	CPPUNIT_TEST( test_nameParsingSimple );
	CPPUNIT_TEST( test_nameParsingComplex );
	CPPUNIT_TEST( test_typeParsingSimple );
	CPPUNIT_TEST( test_typeParsingComplex );
	CPPUNIT_TEST( test_fullParsing );
	CPPUNIT_TEST( test_arrayDefinition );
	CPPUNIT_TEST(test_arrayWithInitialization);
	CPPUNIT_TEST( test_equalVariables );
	CPPUNIT_TEST_SUITE_END();
public:
	
	VariableTest(){}

	void test_nameParsingSimple()
	{
		common::Variable var("int a");
		CPPUNIT_ASSERT(var.getName() == "a");
	}
	void test_nameParsingComplex()
	{
		common::Variable var("float a = -0.5");
		CPPUNIT_ASSERT(var.getName() == "a");
	}

	void test_typeParsingSimple()
	{
		common::Variable var("int x");
		CPPUNIT_ASSERT(var.getType() == "int");
	}

	void test_typeParsingComplex()
	{
		common::Variable var("int x = -453");
		CPPUNIT_ASSERT(var.getType() == "int");
	}

	void test_fullParsing()
	{
		common::Variable var("float _myVariable = 0.4f");
		CPPUNIT_ASSERT(var.getType() == "float");
		CPPUNIT_ASSERT(var.getName() == "_myVariable");
	}

	void test_arrayDefinition()
	{
		common::Variable var("int* myArray[5]");
		CPPUNIT_ASSERT(var.getName() == "myArray");
		CPPUNIT_ASSERT(var.getType() == "int*[5]");
	}

	void test_arrayWithInitialization()
	{
		common::Variable var("int myArray[5] = {0, 1, 2, 3, 4, 5}");
		CPPUNIT_ASSERT(var.getName() == "myArray");
		CPPUNIT_ASSERT(var.getType() == "int[5]");
	}

	void test_equalVariables()
	{
		common::Variable var1("float _myVariable = 0.4f");
		common::Variable var2("float _myVariable = 0.4f");
		CPPUNIT_ASSERT(var1 == var2);
	}

};

