#ifndef ADBProcessor_h__
#define ADBProcessor_h__

#include <map>
#include <vector>
#include <string>

struct StateLines
{
	int lineStart;
	int lineEnd;
	StateLines(): lineStart(0), lineEnd(0){}
	StateLines(int first, int last): lineStart(first), lineEnd(last){}

    StateLines& operator--()
    {
        lineStart--;
        lineEnd--;
        return *this;
    }

    bool isInside(int line) {return line >= lineStart && line <= lineEnd;}
};


struct AutomataStates {
	std::map<int, StateLines> _stateLinesMap;
	std::vector<bool> _stopAtState;
};


struct PDGNode {

    StateLines lines;

    int controlDepsNumber;
    int dataDepsNumber;

    int* dataDependencies;
    int* controlDependencies;
    int stateNumber;

    int graphNumber;

};

struct PDGraph {
    std::string automatonName;
    std::vector<PDGNode> nodes;
};

class ADBProcessor
{
public:

	ADBProcessor():_loadedAndReady(false){};

	bool loadADBFile(const char* fileName);
	StateLines getLinesForState( const std::string& automatonName, int stateNumber);
    PDGNode* getStateForLine(int lineNumber);
	const char* getSourceFileName();

    int getSliceForState(PDGNode& node, StateLines*& slice);

private:

    void _loadPDG(FILE* file, std::string name);

    void _traverseGraph(PDGNode& node, std::vector<StateLines>& slice);

	std::string _sourceFile;
	int _automataNumber;
	std::map<std::string, AutomataStates> _states;

    std::vector<PDGraph*> PDGraphs;

	bool _loadedAndReady;
	
	

};

extern ADBProcessor adbProcessor;


#endif // ADBProcessor_h__