#include <windows.h>
#include <stdio.h>
#include "ScintillaProxy.h"


HINSTANCE scintillaDllInst;
HWND scintillaWnd;

int position = 0, fillLength = 0; //used for selecting current state

#define MAX_FILE_SIZE 1024*1024

int (*scintillaHandlingFunc)(void*,int,int,int);
void * scintillaFirstParameter;
char* text = 0;

static const char g_cppKeyWords[] = 

// Standard

"asm auto bool break case catch char class const "
"const_cast continue default delete do double "
"dynamic_cast else enum explicit extern false finally "
"float for friend goto if inline int long mutable "
"namespace new operator private protected public "
"register reinterpret_cast register return short signed "
"sizeof static static_cast struct switch template "
"this throw true try typedef typeid typename "
"union unsigned using virtual void volatile "
"wchar_t while "

// Extended

"__asm __asume __based __box __cdecl __declspec "
"__delegate delegate depreciated dllexport dllimport "
"event __event __except __fastcall __finally __forceinline "
"__int8 __int16 __int32 __int64 __int128 __interface "
"interface __leave naked noinline __noop noreturn "
"nothrow novtable nullptr safecast __stdcall "
"__try __except __finally __unaligned uuid __uuidof "
"__virtual_inheritance";


/// Scintilla Colors structure

struct SScintillaColors
{   int         iItem;
COLORREF    rgb;
};

// A few basic colors

const COLORREF black = RGB( 0, 0, 0 );
const COLORREF white = RGB( 255, 255, 255 );
const COLORREF green = RGB( 80, 180, 80 );
const COLORREF red = RGB( 200, 50, 50 );
const COLORREF blue = RGB( 0, 0, 255 );
const COLORREF yellow = RGB( 255, 255, 0 );
const COLORREF magenta = RGB( 255, 0, 255 );
const COLORREF cyan = RGB( 0, 255, 255 );
const COLORREF gray = RGB(180, 180, 180);

/// Default color scheme

static SScintillaColors g_rgbSyntaxCpp[] = 
{
	{   SCE_C_COMMENT,          green },
	{   SCE_C_COMMENTLINE,      green },
	{   SCE_C_COMMENTDOC,       green },
	{   SCE_C_NUMBER,           black },
	{   SCE_C_STRING,           red },
	{   SCE_C_CHARACTER,        red },
	{   SCE_C_UUID,             cyan },
	{   SCE_C_OPERATOR,         black },
	{   SCE_C_PREPROCESSOR,     blue },
	{   SCE_C_WORD,             blue },
    {   SCE_C_IDENTIFIER,       gray},
	{   -1,                     0 }
};


bool loadDll()
{
	scintillaDllInst = LoadLibrary("SciLexer.dll");
	if (scintillaDllInst==NULL)
	{
		printf("Scintilla Dll could not be loaded \n");
		return false;
	}
	return true;
}

void unloadScintillaDll()
{
	if (text)
	{
		delete[] text;
	}
	FreeLibrary(scintillaDllInst);
}

void retrieveFunctionPointers(HWND wnd)
{
	scintillaHandlingFunc = (int (__cdecl *)(void *,int,int,int))SendMessage(
		wnd,SCI_GETDIRECTFUNCTION,0,0);
	scintillaFirstParameter = (void *)SendMessage(wnd,SCI_GETDIRECTPOINTER,0,0);
}

void SendEditor(int command, int arg1 = 0, int arg2 = 0)
{
	scintillaHandlingFunc(scintillaFirstParameter, command, arg1, arg2);
}

void SetAStyle(int style, COLORREF fore, COLORREF back = white, int size = -1, const char *face = 0) {
	SendEditor(SCI_STYLESETFORE, style, fore);
	SendEditor(SCI_STYLESETBACK, style, back);
	if (size >= 1)
		SendEditor(SCI_STYLESETSIZE, style, size);
	if (face) 
		SendEditor(SCI_STYLESETFONT, style, reinterpret_cast<LPARAM>(face));
}

void setEditorStyle()
{
	SendEditor( SCI_SETLEXER, SCLEX_CPP );

	// Set number of style bits to use

	SendEditor( SCI_SETSTYLEBITS, 5 );

	// Set tab width

	SendEditor( SCI_SETTABWIDTH, 4 );

	// Use CPP keywords

	SendEditor( SCI_SETKEYWORDS, 0, (LPARAM)g_cppKeyWords );

	// Set up the global default style. These attributes are used wherever no explicit choices are made.

	SetAStyle( STYLE_DEFAULT, black, white, 10, "Courier New" );

		// Set caret foreground color

	SendEditor( SCI_SETCARETFORE, RGB( 255, 255, 255 ) );

	// Set all styles

	SendEditor( SCI_STYLECLEARALL );

	// Set selection color

	SendEditor( SCI_SETSELBACK, TRUE, RGB( 120, 120, 200 ) );


	SendEditor(SCI_INDICSETSTYLE, 3, INDIC_ROUNDBOX);
	//Set color for current state outlining
	SendEditor(SCI_INDICSETFORE, 3, 0xff0000);


    SendEditor(SCI_INDICSETSTYLE,4 , INDIC_ROUNDBOX);
    //Set color for slicing outlining
    SendEditor(SCI_INDICSETFORE, 4, 0x888888);

	// Set syntax colors

	for ( long i = 0; g_rgbSyntaxCpp[ i ].iItem != -1; i++ ) 
		SetAStyle( g_rgbSyntaxCpp[ i ].iItem, g_rgbSyntaxCpp[ i ].rgb );

}

void loadSourceFile(const char* filename)
{
	
	text = new char [MAX_FILE_SIZE];
	FILE* f = fopen(filename,"r");
	if (!f)
	{
		printf("Source file loading failed \n");
		return;
	}
	int trueSize = fread(text, sizeof(char), MAX_FILE_SIZE, f);
	if (trueSize == 0)
	{
		printf("File is empty \n");
		return;
	}
	
	fclose(f);
	scintillaHandlingFunc(scintillaFirstParameter, SCI_ADDTEXT, trueSize , (int)text);
	

}

void setCurrentStateLines(int firstLine, int lastLine)
{
	
	SendEditor(SCI_SETINDICATORCURRENT, 3);
	SendEditor(SCI_INDICATORCLEARRANGE, position, fillLength);
	position = scintillaHandlingFunc(scintillaFirstParameter, SCI_POSITIONFROMLINE, firstLine, 0);
	fillLength = scintillaHandlingFunc(scintillaFirstParameter, SCI_POSITIONFROMLINE, lastLine, 0) - position;
	fillLength += scintillaHandlingFunc(scintillaFirstParameter, SCI_LINELENGTH, lastLine, 0);
	
	
	SendEditor(SCI_INDICATORFILLRANGE, position,  fillLength);
}

void selectLines( int firstLine, int lastLine )
{
    SendEditor(SCI_SETINDICATORCURRENT, 4);
    int position, fillLength;   
    position = scintillaHandlingFunc(scintillaFirstParameter, SCI_POSITIONFROMLINE, firstLine, 0);
    fillLength = scintillaHandlingFunc(scintillaFirstParameter, SCI_POSITIONFROMLINE, lastLine, 0) - position;
    fillLength += scintillaHandlingFunc(scintillaFirstParameter, SCI_LINELENGTH, lastLine, 0);


    SendEditor(SCI_INDICATORFILLRANGE, position,  fillLength);
}

void repaintText()
{
    int textLength = scintillaHandlingFunc(scintillaFirstParameter, SCI_GETLENGTH, 0, 0);
    SendEditor(SCI_SETINDICATORCURRENT, 4);
    SendEditor(SCI_INDICATORCLEARRANGE, 0, textLength);
}