#ifndef ADBProcessor_h__
#define ADBProcessor_h__

#include <map>
#include <vector>
#include <string>

//////////////////////////////////////////////////////////////////////////
//
// Location of some code in input file
//
//

struct StateLines
{
    int lineStart;
    int lineEnd;

    StateLines(): lineStart(0), lineEnd(0){}
    StateLines(int first, int last): lineStart(first), lineEnd(last){}

    StateLines& operator--()
    {
        lineStart--;
        lineEnd--;
        return *this;
    }

    bool isInside(int line) {return line >= lineStart && line <= lineEnd;}
};

typedef std::map<int, StateLines> AutomataStates;

//////////////////////////////////////////////////////////////////////////
///
/// Node of Program Dependency Graph
///
struct PDGNode {

    StateLines lines;

    int controlDepsNumber;
    int dataDepsNumber;

    int* dataDependencies;
    int* controlDependencies;

    int stateNumber;
    int graphNumber;
};

//////////////////////////////////////////////////////////////////////////
///
/// Program Dependency Graph representation
///
struct PDGraph {
    std::string automatonName;
    std::vector<PDGNode> nodes;
};

//////////////////////////////////////////////////////////////////////////
///
/// Loads and processes adb files
///

class ADBProcessor  {
public:

    ADBProcessor():_loadedAndReady(false){};
    ~ADBProcessor() { unloadADBFile(); }

    bool loadADBFile(const char* fileName);
    void unloadADBFile();

    StateLines getLinesForState( const std::string& automatonName, int stateNumber);
    PDGNode* getStateForLine(int lineNumber);
    const char* getSourceFileName();

    int getSliceForState(PDGNode& node, StateLines*& slice);

private:

    void _loadPDG(FILE* file, std::string name);

    void _traverseGraph(PDGNode& node, std::vector<StateLines>& slice);

    std::string _sourceFile;
    int _automataNumber;
    std::map<std::string, AutomataStates> _states;

    std::vector<PDGraph*> PDGraphs;

    bool _loadedAndReady;



};

extern ADBProcessor adbProcessor;



#endif // ADBProcessor_h__